@@ -31,6 +31,7 @@ from kazoo.protocol.serialization import (
     CheckVersion,
     CloseInstance,
     Create,
+    Create2,
     Delete,
     Exists,
     GetChildren,
@@ -68,9 +69,12 @@ from kazoo.recipe.watchers import ChildrenWatch, DataWatch
 string_types = six.string_types
 bytes_types = (six.binary_type,)
 
-LOST_STATES = (KeeperState.EXPIRED_SESSION, KeeperState.AUTH_FAILED,
-               KeeperState.CLOSED)
-ENVI_VERSION = re.compile('([\d\.]*).*', re.DOTALL)
+CLOSED_STATES = (
+    KeeperState.EXPIRED_SESSION,
+    KeeperState.AUTH_FAILED,
+    KeeperState.CLOSED
+)
+ENVI_VERSION = re.compile(r'([\d\.]*).*', re.DOTALL)
 ENVI_VERSION_KEY = 'zookeeper.version'
 log = logging.getLogger(__name__)
 
@@ -102,8 +106,8 @@ class KazooClient(object):
     """
     def __init__(self, hosts='127.0.0.1:2181',
                  timeout=10.0, client_id=None, handler=None,
-                 default_acl=None, auth_data=None, read_only=None,
-                 randomize_hosts=True, connection_retry=None,
+                 default_acl=None, auth_data=None, sasl_options=None,
+                 read_only=None, randomize_hosts=True, connection_retry=None,
                  command_retry=None, logger=None, keyfile=None,
                  keyfile_password=None, certfile=None, ca=None,
                  use_ssl=False, verify_certs=True, **kwargs):
@@ -123,6 +127,31 @@ class KazooClient(object):
             A list of authentication credentials to use for the
             connection. Should be a list of (scheme, credential)
             tuples as :meth:`add_auth` takes.
+        :param sasl_options:
+            SASL options for the connection, if SASL support is to be used.
+            Should be a dict of SASL options passed to the underlying
+            `pure-sasl <https://pypi.org/project/pure-sasl>`_ library.
+
+            For example using the DIGEST-MD5 mechnism:
+
+            .. code-block:: python
+
+                sasl_options = {
+                    'mechanism': 'DIGEST-MD5',
+                    'username': 'myusername',
+                    'password': 'mypassword'
+                }
+
+            For GSSAPI, using the running process' ticket cache:
+
+            .. code-block:: python
+
+                sasl_options = {
+                    'mechanism': 'GSSAPI',
+                    'service': 'myzk',                  # optional
+                    'principal': 'client@EXAMPLE.COM'   # optional
+                }
+
         :param read_only: Allow connections to read only servers.
         :param randomize_hosts: By default randomize host selection.
         :param connection_retry:
@@ -174,6 +203,9 @@ class KazooClient(object):
         .. versionadded:: 1.2
             The connection_retry, command_retry and logger options.
 
+        .. versionadded:: 2.7
+            The sasl_options option.
+
         """
         self.logger = logger or log
 
@@ -273,9 +305,39 @@ class KazooClient(object):
                     sleep_func=self.handler.sleep_func,
                     **retry_keys)
 
+        # Managing legacy SASL options
+        for scheme, auth in self.auth_data:
+            if scheme != 'sasl':
+                continue
+            if sasl_options:
+                raise ConfigurationError(
+                    'Multiple SASL configurations provided'
+                )
+            warnings.warn(
+                'Passing SASL configuration as part of the auth_data is '
+                'deprecated, please use the sasl_options configuration '
+                'instead', DeprecationWarning, stacklevel=2
+            )
+            username, password = auth.split(':')
+            # Generate an equivalent SASL configuration
+            sasl_options = {
+                'username': username,
+                'password': password,
+                'mechanism': 'DIGEST-MD5',
+                'service': 'zookeeper',
+                'principal': 'zk-sasl-md5',
+            }
+        # Cleanup
+        self.auth_data = set([
+            (scheme, auth)
+            for scheme, auth in self.auth_data
+            if scheme != 'sasl'
+        ])
+
         self._conn_retry.interrupt = lambda: self._stopped.is_set()
         self._connection = ConnectionHandler(
-            self, self._conn_retry.copy(), logger=self.logger)
+            self, self._conn_retry.copy(), logger=self.logger,
+            sasl_options=sasl_options)
 
         # Every retry call should have its own copy of the retry helper
         # to avoid shared retry counts
@@ -303,15 +365,6 @@ class KazooClient(object):
         self.Semaphore = partial(Semaphore, self)
         self.ShallowParty = partial(ShallowParty, self)
 
-        # Managing SASL client
-        self.use_sasl = False
-        for scheme, auth in self.auth_data:
-            if scheme == "sasl":
-                self.use_sasl = True
-                # Could be used later for GSSAPI implementation
-                self.sasl_server_principal = "zk-sasl-md5"
-                break
-
         # If we got any unhandled keywords, complain like Python would
         if kwargs:
             raise TypeError('__init__() got unexpected keyword arguments: %s'
@@ -464,14 +517,14 @@ class KazooClient(object):
 
         # Note that we don't check self.state == LOST since that's also
         # the client's initial state
-        dead_state = self._state in LOST_STATES
+        closed_state = self._state in CLOSED_STATES
         self._state = state
 
         # If we were previously closed or had an expired session, and
         # are now connecting, don't bother with the rest of the
         # transitions since they only apply after
         # we've established a connection
-        if dead_state and state == KeeperState.CONNECTING:
+        if closed_state and state == KeeperState.CONNECTING:
             self.logger.log(BLATHER, "Skipping state change")
             return
 
@@ -480,8 +533,8 @@ class KazooClient(object):
                              "state: %s", state)
             self._live.set()
             self._make_state_change(KazooState.CONNECTED)
-        elif state in LOST_STATES:
-            self.logger.info("Zookeeper session lost, state: %s", state)
+        elif state in CLOSED_STATES:
+            self.logger.info("Zookeeper session closed, state: %s", state)
             self._live.clear()
             self._make_state_change(KazooState.LOST)
             self._notify_pending(state)
@@ -531,11 +584,11 @@ class KazooClient(object):
                 "and wouldn't close after %s seconds" % timeout)
 
     def _call(self, request, async_object):
-        """Ensure there's an active connection and put the request in
-        the queue if there is.
+        """Ensure the client is in CONNECTED or SUSPENDED state and put the
+        request in the queue if it is.
 
         Returns False if the call short circuits due to AUTH_FAILED,
-        CLOSED, EXPIRED_SESSION or CONNECTING state.
+        CLOSED, or EXPIRED_SESSION state.
 
         """
 
@@ -546,8 +599,7 @@ class KazooClient(object):
             async_object.set_exception(ConnectionClosedError(
                 "Connection has been closed"))
             return False
-        elif self._state in (KeeperState.EXPIRED_SESSION,
-                             KeeperState.CONNECTING):
+        elif self._state == KeeperState.EXPIRED_SESSION:
             async_object.set_exception(SessionExpiredError())
             return False
 
@@ -560,7 +612,7 @@ class KazooClient(object):
                 "Connection has been closed"))
         try:
             write_sock.send(b'\0')
-        except:
+        except:  # NOQA
             async_object.set_exception(ConnectionClosedError(
                 "Connection has been closed"))
 
@@ -579,6 +631,7 @@ class KazooClient(object):
         if not self.connected:
             # We time-out, ensure we are disconnected
             self.stop()
+            self.close()
             raise self.handler.timeout_exception("Connection time-out")
 
         if self.chroot and not self.exists("/"):
@@ -737,12 +790,8 @@ class KazooClient(object):
         """Send credentials to server.
 
         :param scheme: authentication scheme (default supported:
-                       "digest", "sasl"). Note that "sasl" scheme is
-                       requiring "pure-sasl" library to be
-                       installed.
+                       "digest").
         :param credential: the credential -- value depends on scheme.
-                           "digest": user:password
-                           "sasl": user:password
 
         :returns: True if it was successful.
         :rtype: bool
@@ -777,7 +826,8 @@ class KazooClient(object):
         """Strip the chroot if applicable from the path."""
         if not self.chroot:
             return path
-
+        if self.chroot == path:
+            return "/"
         if path.startswith(self.chroot):
             return path[len(self.chroot):]
         else:
@@ -790,7 +840,20 @@ class KazooClient(object):
 
         """
         async_result = self.handler.async_result()
-        self._call(Sync(_prefix_root(self.chroot, path)), async_result)
+
+        @wrap(async_result)
+        def _sync_completion(result):
+            return self.unchroot(result.get())
+
+        def _do_sync():
+            result = self.handler.async_result()
+            self._call(
+                Sync(_prefix_root(self.chroot, path)),
+                result
+            )
+            result.rawlink(_sync_completion)
+
+        _do_sync()
         return async_result
 
     def sync(self, path):
@@ -810,7 +873,7 @@ class KazooClient(object):
         return self.sync_async(path).get()
 
     def create(self, path, value=b"", acl=None, ephemeral=False,
-               sequence=False, makepath=False):
+               sequence=False, makepath=False, include_data=False):
         """Create a node with the given value as its data. Optionally
         set an ACL on the node.
 
@@ -858,7 +921,13 @@ class KazooClient(object):
                          with a unique index.
         :param makepath: Whether the path should be created if it
                          doesn't exist.
-        :returns: Real path of the new node.
+        :param include_data:
+            Include the :class:`~kazoo.protocol.states.ZnodeStat` of
+            the node in addition to its real path. This option changes
+            the return value to be a tuple of (path, stat).
+
+        :returns: Real path of the new node, or tuple if `include_data`
+                  is `True`
         :rtype: str
 
         :raises:
@@ -877,13 +946,19 @@ class KazooClient(object):
             :exc:`~kazoo.exceptions.ZookeeperError` if the server
             returns a non-zero error code.
 
+        .. versionadded:: 1.1
+            The `makepath` option.
+        .. versionadded:: 2.7
+            The `include_data` option.
         """
         acl = acl or self.default_acl
-        return self.create_async(path, value, acl=acl, ephemeral=ephemeral,
-                                 sequence=sequence, makepath=makepath).get()
+        return self.create_async(
+            path, value, acl=acl, ephemeral=ephemeral,
+            sequence=sequence, makepath=makepath, include_data=include_data
+        ).get()
 
     def create_async(self, path, value=b"", acl=None, ephemeral=False,
-                     sequence=False, makepath=False):
+                     sequence=False, makepath=False, include_data=False):
         """Asynchronously create a ZNode. Takes the same arguments as
         :meth:`create`.
 
@@ -891,7 +966,8 @@ class KazooClient(object):
 
         .. versionadded:: 1.1
             The makepath option.
-
+        .. versionadded:: 2.7
+            The `include_data` option.
         """
         if acl is None and self.default_acl:
             acl = self.default_acl
@@ -910,6 +986,8 @@ class KazooClient(object):
             raise TypeError("Invalid type for 'sequence' (bool expected)")
         if not isinstance(makepath, bool):
             raise TypeError("Invalid type for 'makepath' (bool expected)")
+        if not isinstance(include_data, bool):
+            raise TypeError("Invalid type for 'include_data' (bool expected)")
 
         flags = 0
         if ephemeral:
@@ -924,7 +1002,9 @@ class KazooClient(object):
         @capture_exceptions(async_result)
         def do_create():
             result = self._create_async_inner(
-                path, value, acl, flags, trailing=sequence)
+                path, value, acl, flags,
+                trailing=sequence, include_data=include_data
+            )
             result.rawlink(create_completion)
 
         @capture_exceptions(async_result)
@@ -935,7 +1015,11 @@ class KazooClient(object):
         @wrap(async_result)
         def create_completion(result):
             try:
-                return self.unchroot(result.get())
+                if include_data:
+                    new_path, stat = result.get()
+                    return self.unchroot(new_path), stat
+                else:
+                    return self.unchroot(result.get())
             except NoNodeError:
                 if not makepath:
                     raise
@@ -948,10 +1032,16 @@ class KazooClient(object):
         do_create()
         return async_result
 
-    def _create_async_inner(self, path, value, acl, flags, trailing=False):
+    def _create_async_inner(self, path, value, acl, flags,
+                            trailing=False, include_data=False):
         async_result = self.handler.async_result()
+        if include_data:
+            opcode = Create2
+        else:
+            opcode = Create
+
         call_result = self._call(
-            Create(_prefix_root(self.chroot, path, trailing=trailing),
+            opcode(_prefix_root(self.chroot, path, trailing=trailing),
                    value, acl, flags), async_result)
         if call_result is False:
             # We hit a short-circuit exit on the _call. Because we are
