@@ -12,25 +12,25 @@ environments that use threads.
 """
 from __future__ import absolute_import
 
+from collections import defaultdict
 import errno
+from itertools import chain
 import logging
 import select
 import socket
 import threading
 import time
+
 import six
 
-from collections import defaultdict
-from itertools import chain
-
 import kazoo.python2atexit as python2atexit
+from kazoo.handlers import utils
 
 try:
     import Queue
 except ImportError:  # pragma: nocover
     import queue as Queue
 
-from kazoo.handlers import utils
 
 # sentinel objects
 _STOP = object()
@@ -113,6 +113,10 @@ class SequentialThreadingHandler(object):
         self._state_change = threading.Lock()
         self._workers = []
 
+    @property
+    def running(self):
+        return self._running
+
     def _create_thread_worker(self, queue):
         def _thread_worker():  # pragma: nocover
             while True:
@@ -126,6 +130,7 @@ class SequentialThreadingHandler(object):
                         log.exception("Exception in worker queue thread")
                     finally:
                         queue.task_done()
+                        del func  # release before possible idle
                 except self.queue_empty:
                     continue
         t = self.spawn(_thread_worker)
@@ -173,7 +178,7 @@ class SequentialThreadingHandler(object):
         # anything to minimize changes
         if _HAS_EPOLL:
             # if the highest fd we've seen is > 1023
-            if max(map(_to_fileno, chain(*args[:3]))) > 1023:
+            if max(map(_to_fileno, chain.from_iterable(args[:3]))) > 1023:
                 return self._epoll_select(*args, **kwargs)
         return self._select(*args, **kwargs)
 
