@@ -1,8 +1,6 @@
 import unittest
 
-from nose import SkipTest
-from nose.tools import eq_
-from nose.tools import raises
+import pytest
 
 from kazoo.client import KazooClient
 from kazoo.exceptions import NoNodeError
@@ -12,23 +10,25 @@ from kazoo.tests import test_client
 
 
 class TestGeventHandler(unittest.TestCase):
-
     def setUp(self):
         try:
             import gevent  # NOQA
         except ImportError:
-            raise SkipTest('gevent not available.')
+            pytest.skip("gevent not available.")
 
     def _makeOne(self, *args):
         from kazoo.handlers.gevent import SequentialGeventHandler
+
         return SequentialGeventHandler(*args)
 
     def _getAsync(self, *args):
         from kazoo.handlers.gevent import AsyncResult
+
         return AsyncResult
 
     def _getEvent(self):
         from gevent.event import Event
+
         return Event
 
     def test_proper_threading(self):
@@ -45,10 +45,8 @@ class TestGeventHandler(unittest.TestCase):
     def test_exception_raising(self):
         h = self._makeOne()
 
-        @raises(h.timeout_exception)
-        def testit():
+        with pytest.raises(h.timeout_exception):
             raise h.timeout_exception("This is a timeout")
-        testit()
 
     def test_exception_in_queue(self):
         h = self._makeOne()
@@ -57,14 +55,15 @@ class TestGeventHandler(unittest.TestCase):
 
         def func():
             ev.set()
-            raise ValueError('bang')
+            raise ValueError("bang")
 
-        call1 = Callback('completion', func, ())
+        call1 = Callback("completion", func, ())
         h.dispatch_callback(call1)
         ev.wait()
 
     def test_queue_empty_exception(self):
         from gevent.queue import Empty
+
         h = self._makeOne()
         h.start()
         ev = self._getEvent()()
@@ -73,38 +72,39 @@ class TestGeventHandler(unittest.TestCase):
             ev.set()
             raise Empty()
 
-        call1 = Callback('completion', func, ())
+        call1 = Callback("completion", func, ())
         h.dispatch_callback(call1)
         ev.wait()
 
 
 class TestBasicGeventClient(KazooTestCase):
-
     def setUp(self):
         try:
             import gevent  # NOQA
         except ImportError:
-            raise SkipTest('gevent not available.')
+            pytest.skip("gevent not available.")
         KazooTestCase.setUp(self)
 
     def _makeOne(self, *args):
         from kazoo.handlers.gevent import SequentialGeventHandler
+
         return SequentialGeventHandler(*args)
 
     def _getEvent(self):
         from gevent.event import Event
+
         return Event
 
     def test_start(self):
         client = self._get_client(handler=self._makeOne())
         client.start()
-        self.assertEqual(client.state, 'CONNECTED')
+        assert client.state == "CONNECTED"
         client.stop()
 
     def test_start_stop_double(self):
         client = self._get_client(handler=self._makeOne())
         client.start()
-        self.assertEqual(client.state, 'CONNECTED')
+        assert client.state == "CONNECTED"
         client.handler.start()
         client.handler.stop()
         client.stop()
@@ -112,47 +112,48 @@ class TestBasicGeventClient(KazooTestCase):
     def test_basic_commands(self):
         client = self._get_client(handler=self._makeOne())
         client.start()
-        self.assertEqual(client.state, 'CONNECTED')
-        client.create('/anode', 'fred')
-        eq_(client.get('/anode')[0], 'fred')
-        eq_(client.delete('/anode'), True)
-        eq_(client.exists('/anode'), None)
+        assert client.state == "CONNECTED"
+        client.create("/anode", b"fred")
+        assert client.get("/anode")[0] == b"fred"
+        assert client.delete("/anode")
+        assert client.exists("/anode") is None
         client.stop()
 
     def test_failures(self):
         client = self._get_client(handler=self._makeOne())
         client.start()
-        self.assertRaises(NoNodeError, client.get, '/none')
+        with pytest.raises(NoNodeError):
+            client.get("/none")
         client.stop()
 
     def test_data_watcher(self):
         client = self._get_client(handler=self._makeOne())
         client.start()
-        client.ensure_path('/some/node')
+        client.ensure_path("/some/node")
         ev = self._getEvent()()
 
-        @client.DataWatch('/some/node')
+        @client.DataWatch("/some/node")
         def changed(d, stat):
             ev.set()
 
         ev.wait()
         ev.clear()
-        client.set('/some/node', 'newvalue')
+        client.set("/some/node", b"newvalue")
         ev.wait()
         client.stop()
 
 
 class TestGeventClient(test_client.TestClient):
-
     def setUp(self):
         try:
             import gevent  # NOQA
         except ImportError:
-            raise SkipTest('gevent not available.')
+            pytest.skip("gevent not available.")
         KazooTestCase.setUp(self)
 
     def _makeOne(self, *args):
         from kazoo.handlers.gevent import SequentialGeventHandler
+
         return SequentialGeventHandler(*args)
 
     def _get_client(self, **kwargs):
